
/*===============================<o>=====================================

  This software was written by Ian L. Kaplan, Chief Fat Bear, Bear Products
  International.  Use of this software, for any purpose, is granted on two
  conditions: 

   1. This copyright notice must be included with the software
      or any software derived from it.

   2. The risk of using this software is accepted by the user.  No
      warranty as to its usefulness or functionality is provided
      or implied.  The author and Bear Products International provides
      no support.

   May 1996

 ===============================<o>=====================================*/

#include "dxfvals.h"
#include "dxfwrite.h"

//
// dxfwrite class functions to write out the Sierpinski pyramid
// point list in DXF file format, using the 3DFACE entity.
//

//
// DXF name for the entity (e.g., the 3D shape).
//
const char *EntityName = "Sier";


//
// write_longval
//
// Write an unsigned integer value in a seven character field.
// This is used to write DXF flag values.
//
void dxfwrite::write_longval(unsigned int val)
{
  fprintf(fp, "%7d\n", val );
}


//
// write_dxfval
//
// Write a DXF value in a three character field.  This is used
// to write out most of the DXF markers (e.g., like start, name, etc...).
//
void dxfwrite::write_dxfval(unsigned int val)
{
  fprintf(fp, "%3d\n", val );
}  // write_dxfval



void dxfwrite::write_dxfstring(const char *name)
{
  fprintf(fp, "%s\n", name);
}


typedef struct { char *GLname;
  dxftoken token;
} table_rec;

static table_rec res_word_table[] = {  {"bad_dxftoken", bad_dxftoken},
				       {"SECTION", section},
				       {"HEADER", header },
				       {"ENDSEC", endsec },
				       {"SEQEND", seqend },
				       {"BLOCKS", blocks },
				       {"BLOCK",  block  },
				       {"ENDBLK", endblk },
				       {"VERTEX", vertex },
				       {"TABLES", tables },
				       {"TABLE", table },
				       {"LAYER", layer },
				       {"CONTINUOUS", contin },
				       {"ENDTAB", endtab },
				       {"ENTITIES", entities},
				       {"3DFACE", three_d_face},
				       {"EOF", end_of_file },
				       {"POLYLINE", polyline },
				       {"integer", integer},
				       {"real", real},
				       {"name", name},
				       { NULL, bad_dxftoken }};




void dxfwrite::write_dxfname(const dxftoken token)
{
  char *str = "unknown";

  if (token < last_dxftoken)
    str = res_word_table[token].GLname;
  fprintf(fp, "%s\n", str );
} // print_token



//
// Write the DXF file header.  This header is in the following form:
//
//    0
//  SECTION
//    2
//  HEADER
//    0
//  ENDSEC
//
void dxfwrite::write_header(void)
{
  write_dxfval( DXF_start );
  write_dxfname( section );
  write_dxfval( DXF_name );
  write_dxfname( header );
  write_dxfval( DXF_start );
  write_dxfname( endsec );
}  // write_header


//
// write_tables
//
// Write out the DXF "tables" section
//
//    0
//  SECTION
//    2
//  TABLES
//    0
//  TABLE
//    2
//  LAYER
//    70
//      153
//    0
//  LAYER
//    2
//  Cube3
//   70
//  0
//   62
//  15
//    6
//  CONTINUOUS
//    0
//  ENDTAB
//    0
//  ENDSEC
//
void dxfwrite::write_tables(void)
{

  write_dxfval( DXF_start );            //     0
  write_dxfname( section );             //  SECTION
  write_dxfval( DXF_name );             //     2
  write_dxfname( tables );              //  TABLES
  write_dxfval( DXF_start );            //     0
  write_dxfname( table );               //  TABLE
  write_dxfval( DXF_name );             //     2
  write_dxfname( layer );               //  LAYER
  write_dxfval( DXF_SeventyFlag );      //    70
  write_longval( 153 );                 //     153
  write_dxfval( DXF_start );            //    0
  write_dxfname( layer );               //  LAYER
  write_dxfval( DXF_name );             //     2
  write_dxfstring( EntityName );        //   Sier
  write_dxfval( DXF_SeventyFlag );      //    70
  write_dxfval( 0 );                    //   0
  write_dxfval( DXF_colornum );         //    62
  write_dxfval( 15 );                   //   15
  write_longval( 6 );                   //     6
  write_dxfname( contin );              //  CONTINUOUS
  write_dxfval( DXF_start );            //    0
  write_dxfname( endtab );              //  ENDTAB
  write_dxfval( DXF_start );            //    0
  write_dxfname( endsec );              //  ENDSEC
}  // write_tables


// write_entstart
//
// Write the DXF file entities start section
//
//    0
//  SECTION
//    2
//  ENTITIES
//
void dxfwrite::write_entstart(void)
{
  write_dxfval( DXF_start );            //    0
  write_dxfname( section );             //  SECTION
  write_dxfval( DXF_name );             //     2
  write_dxfname( entities );            //  ENTITIES
}  // write_entstart


void dxfwrite::write_3DFACE_start(void)
{
  write_dxfval( DXF_start );            //    0
  write_dxfname( three_d_face );        //  3DFACE
  write_dxfval( DXF_layer_name );       //    8
  write_dxfstring( EntityName );        //   Sier
}



void dxfwrite::write_endsec(void)
{
  write_dxfval( DXF_start );            //    0
  write_dxfname( endsec );              //  ENDSEC
}


void dxfwrite::write_EOF(void)
{
  write_dxfval( DXF_start );            //    0
  write_dxfname( end_of_file );         //  EOF
}


void dxfwrite::write_blankcolornum(void)
{
  write_dxfval( DXF_colornum );         //   62
  write_dxfval( 0 );                    //  0
}



void dxfwrite::write_dxfcoord(int mark, int *coord )
{
  fprintf(fp, "%3d\n", mark);
  fprintf(fp, "%d.000000\n", *coord );
}



void dxfwrite::write_dxfpoint(int *pointlist)
{
  int i;
  int next_coordmark;
  
  if (coordmark == 13)
    next_coordmark = 10;
  else
    next_coordmark = coordmark + 1;

  for (i = 0; i < 3; i++) {
    write_dxfcoord(coordmark, pointlist );
    coordmark += 10;
    pointlist++;
  }
  coordmark = next_coordmark;
}



void dxfwrite::write_faces(int *pointlist, int num_faces)
{
  typedef int point[ 3 ];
  const int size_of_point = sizeof( point )/sizeof(int);
  const int points_per_face = 4;
  int i, j;

  for (j = 0; j < num_faces; j++) {
    write_3DFACE_start();
    for (i = 0; i < points_per_face; i++) {
      write_dxfpoint( pointlist );
      pointlist += size_of_point;
    } // i
    write_blankcolornum();
  } // j
}  // write_faces



void dxfwrite::write_3DFACE( FILE *fileptr, int *pointlist, int num_faces )
{
  fp = fileptr;
  write_header();
  write_tables();
  write_entstart();
  write_faces(pointlist, num_faces);
  write_endsec();
  write_EOF();
} // write_3DFACE


